function [NODE,ELEM,SUPP,LOAD] = StructDomain(Nx,Ny,Lx,Ly,ProblemID)
narginchk(4,5)

% Generate structured-orthogonal domains
[X,Y] = meshgrid(linspace(0,Lx,Nx+1),linspace(0,Ly,Ny+1));
NODE = [reshape(X,numel(X),1) reshape(Y,numel(Y),1)];
Nn = (Nx+1)*(Ny+1); Ne = Nx*Ny;
ELEM = cell(Ne,1);
k = 0;
for j=1:Ny
    for i=1:Nx
        k = k + 1;
        n1 = (i-1)*(Ny+1)+j; n2 = i*(Ny+1)+j;
        ELEM{k} = [n1 n2 n2+1 n1+1];
    end
end

if (nargin==4 || isempty(ProblemID)), ProblemID = 1; end
switch lower(ProblemID)
    case {'cantilever',1}
        SUPP = [(1:Ny+1)' ones(Ny+1,2)];
        LOAD = [Nx*(Ny+1)+round((Ny+1)/2) 0 -1];
    case {'mbb',2}
        SUPP = [Nx*(Ny+1)+1 NaN 1;
                (1:Ny+1)' ones(Ny+1,1) nan(Ny+1,1)];
        LOAD = [Ny+1 0 -0.5];
    case {'bridge',3}
        SUPP = [     1      1 1;
                Nx*(Ny+1)+1 1 1];
        LOAD = [(Ny+1)*round(Nx/2)+1 0 -1];
    case {'tower',4}
        SUPP = [(1:(Ny+1):Nn)' zeros(Nx+1,2)];
        loadF = [0.5 ones(1,Ny-1) 0.5]' / Ny; % Normalized load (1-side)
        LOAD = [     (1:Ny+1)'      loadF zeros(Ny+1,1);
                Nx*(Ny+1)+(1:Ny+1)' loadF zeros(Ny+1,1)];
    otherwise
        SUPP = []; LOAD = [];
        fprintf('-INFO- Structured domain generated with no loads/BC\n')
end
return